@@ -1,6 +1,9 @@
 from __future__ import annotations
 
+import sys
+
 from typing import TYPE_CHECKING
+from typing import ClassVar
 
 from cleo.helpers import argument
 from cleo.helpers import option
@@ -11,6 +14,8 @@ from poetry.console.commands.group_command import GroupCommand
 
 
 if TYPE_CHECKING:
+    from cleo.io.inputs.argument import Argument
+    from cleo.io.inputs.option import Option
     from cleo.io.io import IO
     from cleo.ui.table import Rows
     from packaging.utils import NormalizedName
@@ -36,20 +41,18 @@ class ShowCommand(GroupCommand, EnvCommand):
     name = "show"
     description = "Shows information about packages."
 
-    arguments = [argument("package", "The package to inspect", optional=True)]
-    options = [
+    arguments: ClassVar[list[Argument]] = [
+        argument("package", "The package to inspect", optional=True)
+    ]
+    options: ClassVar[list[Option]] = [
         *GroupCommand._group_dependency_options(),
-        option(
-            "no-dev",
-            None,
-            "Do not list the development dependencies. (<warning>Deprecated</warning>)",
-        ),
         option("tree", "t", "List the dependencies as a tree."),
         option(
             "why",
             None,
             "When showing the full list, or a <info>--tree</info> for a single package,"
-            " also display why it's included.",
+            " display whether they are a direct dependency or required by other"
+            " packages",
         ),
         option("latest", "l", "Show the latest version."),
         option(
@@ -62,12 +65,18 @@ class ShowCommand(GroupCommand, EnvCommand):
             "a",
             "Show all packages (even those not compatible with current system).",
         ),
+        option("top-level", "T", "Show only top-level dependencies."),
+        option(
+            "no-truncate",
+            None,
+            "Do not truncate the output based on the terminal width.",
+        ),
     ]
 
     help = """The show command displays detailed information about a package, or
 lists all packages available."""
 
-    colors = ["cyan", "yellow", "green", "magenta", "blue"]
+    colors: ClassVar[list[str]] = ["cyan", "yellow", "green", "magenta", "blue"]
 
     def handle(self) -> int:
         package = self.argument("package")
@@ -75,6 +84,20 @@ lists all packages available."""
         if self.option("tree"):
             self.init_styles(self.io)
 
+        if self.option("top-level"):
+            if self.option("tree"):
+                self.line_error(
+                    "<error>Error: Cannot use --tree and --top-level at the same"
+                    " time.</error>"
+                )
+                return 1
+            if package is not None:
+                self.line_error(
+                    "<error>Error: Cannot use --top-level when displaying a single"
+                    " package.</error>"
+                )
+                return 1
+
         if self.option("why"):
             if self.option("tree") and package is None:
                 self.line_error(
@@ -140,10 +163,7 @@ lists all packages available."""
                 packages = [pkg]
                 if required_by:
                     packages = [
-                        p
-                        for p in locked_packages
-                        for r in required_by.keys()
-                        if p.name == r
+                        p for p in locked_packages for r in required_by if p.name == r
                     ]
                 else:
                     # if no rev-deps exist we'll make this clear as it can otherwise
@@ -182,7 +202,7 @@ lists all packages available."""
             self.line("")
             self.line("<info>required by</info>")
             for parent, requires_version in required_by.items():
-                self.line(f" - <c1>{parent}</c1> <b>{requires_version}</b>")
+                self.line(f" - <c1>{parent}</c1> requires <b>{requires_version}</b>")
 
         return 0
 
@@ -199,8 +219,7 @@ lists all packages available."""
         from poetry.utils.helpers import get_package_version_display_string
 
         locked_packages = locked_repository.packages
-        pool = RepositoryPool(ignore_repository_names=True)
-        pool.add_repository(locked_repository)
+        pool = RepositoryPool.from_packages(locked_packages, self.poetry.config)
         solver = Solver(
             root,
             pool=pool,
@@ -216,7 +235,12 @@ lists all packages available."""
 
         show_latest = self.option("latest")
         show_all = self.option("all")
-        width = shutil.get_terminal_size().columns
+        show_top_level = self.option("top-level")
+        width = (
+            sys.maxsize
+            if self.option("no-truncate")
+            else shutil.get_terminal_size().columns
+        )
         name_length = version_length = latest_length = required_by_length = 0
         latest_packages = {}
         latest_statuses = {}
@@ -242,9 +266,9 @@ lists all packages available."""
                     latest = locked
 
                 latest_packages[locked.pretty_name] = latest
-                update_status = latest_statuses[
-                    locked.pretty_name
-                ] = self.get_update_status(latest, locked)
+                update_status = latest_statuses[locked.pretty_name] = (
+                    self.get_update_status(latest, locked)
+                )
 
                 if not self.option("outdated") or update_status != "up-to-date":
                     name_length = max(name_length, current_length)
@@ -252,7 +276,7 @@ lists all packages available."""
                         version_length,
                         len(
                             get_package_version_display_string(
-                                locked, root=self.poetry.file.parent
+                                locked, root=self.poetry.file.path.parent
                             )
                         ),
                     )
@@ -260,7 +284,7 @@ lists all packages available."""
                         latest_length,
                         len(
                             get_package_version_display_string(
-                                latest, root=self.poetry.file.parent
+                                latest, root=self.poetry.file.path.parent
                             )
                         ),
                     )
@@ -277,7 +301,7 @@ lists all packages available."""
                     version_length,
                     len(
                         get_package_version_display_string(
-                            locked, root=self.poetry.file.parent
+                            locked, root=self.poetry.file.path.parent
                         )
                     ),
                 )
@@ -297,10 +321,16 @@ lists all packages available."""
         write_why = self.option("why") and (why_end_column + 3) <= width
         write_description = (why_end_column + 24) <= width
 
+        requires = root.all_requires
+
         for locked in locked_packages:
             color = "cyan"
             name = locked.pretty_name
             install_marker = ""
+
+            if show_top_level and not any(locked.satisfies(r) for r in requires):
+                continue
+
             if locked not in required_locked_packages:
                 if not show_all:
                     continue
@@ -330,7 +360,7 @@ lists all packages available."""
             )
             if write_version:
                 version = get_package_version_display_string(
-                    locked, root=self.poetry.file.parent
+                    locked, root=self.poetry.file.path.parent
                 )
                 line += f" <b>{version:{version_length}}</b>"
             if show_latest:
@@ -345,7 +375,7 @@ lists all packages available."""
                         color = "yellow"
 
                     version = get_package_version_display_string(
-                        latest, root=self.poetry.file.parent
+                        latest, root=self.poetry.file.path.parent
                     )
                     line += f" <fg={color}>{version:{latest_length}}</>"
 
@@ -520,18 +550,25 @@ lists all packages available."""
         from poetry.version.version_selector import VersionSelector
 
         # find the latest version allowed in this pool
+        requires = root.all_requires
         if package.is_direct_origin():
-            requires = root.all_requires
-
             for dep in requires:
                 if dep.name == package.name and dep.source_type == package.source_type:
                     provider = Provider(root, self.poetry.pool, NullIO())
                     return provider.search_for_direct_origin_dependency(dep)
 
+        allow_prereleases: bool | None = None
+        for dep in requires:
+            if dep.name == package.name:
+                allow_prereleases = dep.allows_prereleases()
+                break
+
         name = package.name
         selector = VersionSelector(self.poetry.pool)
 
-        return selector.find_best_candidate(name, f">={package.pretty_version}")
+        return selector.find_best_candidate(
+            name, f">={package.pretty_version}", allow_prereleases
+        )
 
     def get_update_status(self, latest: Package, package: Package) -> str:
         from poetry.core.constraints.version import parse_constraint
